 /*
 * FILE --- veh_sim.cc
 *
 * PURPOSE --- To rpovide an example use of the class for 4'th
 * order Runge-Kutta simulation, RK4_SIM.
 * Implements the functions of a class for performing a simulation of
 * a simple vehicle with point mass dynamics.
 *
 */

// Include Files
#include "veh_sim_ex.hh"

VEH_SIM::VEH_SIM(double _sampling_period, int _simulation_per_cycle,
		 double _initial_time, double *_initial_state) 
  : RK4_SIM(3, 3, (_sampling_period/((double)_simulation_per_cycle)),
	_initial_time, _initial_state)
{
  // samp_period >= integration_step

  // Initialize parameters
  sampling_period = _sampling_period;
  simulation_per_cycle = _simulation_per_cycle;
  mass = 1576;
  drag_x = 0.3;
  drag_y = 0.1;

}

inline void VEH_SIM::DERIV(const double t, const double *x, 
			   const double *u, double *xdot)
{
  // This is where the derivative is defined: xdot = f(t,x,u)
  // Simple point mass dynamics
  // x[0] = longitudinal velocity
  // x[1] = lateral position
  // x[2] = lateral velocity

  xdot[0] = (1.0/mass)*(-drag_x*(x[0] + u[2]) + u[0]*cos(u[1]));
  xdot[1] = x[2];
  xdot[2] = (1.0/mass)*(-drag_y*x[2] + u[0]*sin(u[1]));
}

void VEH_SIM::simulate()
{
  // This function implements simple state-feedback controller
  // It could be implemented in more efficient way, however, I did not
  // care because the purpose is only to provide an example.
  // Note that after calculating each control input the simulation
  // algorithm is called simulation_per_cycle times. 

  // State feedback gains
  double K11 = 4731;  // - 3.0021;
  double K12 = - 1471; // 0.9333;
  double K13 = - 520;  // 0.3297;
  
  double K21 = - 130;  // 0.0822;
  double K22 = 15788; // - 10.0178;
  double K23 = 11028; // - 6.9976;
  
  // Reference states
  double ref_vel = 60.0;
  double ref_lane = RIGHT_LANE;
  double ref_y_pos = 3.0*ref_lane;
  
  // Intermadiate variables
  double u1 = 0;
  double u2 = 0;
  double u11 = 0;
  double u12 = 0;
  double u21 = 0;
  double u22 = 0;
  
  if (plant_state[0] >= ref_vel)
    {
      u11 = 0;
      u21 = 0;
    }
  else
    {
      u11 = K11*(plant_state[0] - ref_vel);
      u21 = K21*(plant_state[0] - ref_vel);
    }
  
  u12 = K12*(plant_state[1] - ref_y_pos) + K13*plant_state[2];
  u22 = K22*(plant_state[1] - ref_y_pos) + K23*plant_state[2];
  
  u1 = u11 + u12;
  u2 = u21 + u22;
  
  ctrl_input[0] = sqrt(u1*u1 + u2*u2);
  ctrl_input[1] = atan(u2/u1);
  ctrl_input[2] = ref_vel;
  
  for (int ii=0; ii < simulation_per_cycle; ii++)
    {
      solve();
    }
}


